import unittest

from mokaplayer.core.fetchers import lyrics as lyricsfetcher

@unittest.skip('')
class FetcherLyricsTest(unittest.TestCase):

    def test_lyrics_fetcher(self):
        success, lyrics = lyricsfetcher.get('Thriller', 'Michael Jackson', 'Thriller')
        self.assertTrue(success)
        self.assertNotRegex(lyrics, 'Lyrics not found')

    def test_provider_musicmatch(self):
        lyrics = lyricsfetcher.ProviderMusicmatch().get_lyrics('Thriller', 'Michael Jackson', 'Thriller')
        self.assertNotEqual(lyrics, '')

    def test_provider_azlyrics(self):
        lyrics = lyricsfetcher.ProviderAzLyrics().get_lyrics('Thriller', 'Michael Jackson', 'Thriller')
        self.assertNotEqual(lyrics, '')

    def test_provider_metrolyrics(self):
        lyrics = lyricsfetcher.ProviderMetrolyrics().get_lyrics('Thriller', 'Michael Jackson', 'Thriller')
        self.assertNotEqual(lyrics, '')

    def test_provider_genius(self):
        lyrics = lyricsfetcher.ProviderGenius().get_lyrics('Thriller', 'Michael Jackson', 'Thriller')
        self.assertNotEqual(lyrics, '')

    def test_provider_darklyrics(self):
        lyrics = lyricsfetcher.ProviderDarkLyrics().get_lyrics('Enter Sandman', 'Metallica', 'Metallica (Black Album)')
        self.assertNotEqual(lyrics, '')



import unittest
from mokaplayer.core.helpers import time_helper


class TimeHelperTest(unittest.TestCase):
    def test_seconds_to_string(self):
        self.assertEqual(time_helper.seconds_to_string(3), '00:03')
        self.assertEqual(time_helper.seconds_to_string(60), '01:00')
        self.assertEqual(time_helper.seconds_to_string(62), '01:02')
        self.assertEqual(time_helper.seconds_to_string(78), '01:18')
        self.assertEqual(time_helper.seconds_to_string(None), '00:00')
        self.assertEqual(time_helper.seconds_to_string(0), '00:00')



import shutil
import unittest
import pathlib

from mokaplayer.core.fetchers import artworks as artworksfetcher
from mokaplayer.config import appconfig

@unittest.skip('')
class FetcherArtworkTest(unittest.TestCase):

    FOLDER = './test/artworks'

    def setUp(self):
        pathlib.Path(FetcherArtworkTest.FOLDER).mkdir(exist_ok=True)
        (pathlib.Path(FetcherArtworkTest.FOLDER) / 'test.png').touch()

    def tearDown(self):
        shutil.rmtree(FetcherArtworkTest.FOLDER)

    def test_artwork_fetcher(self):
        artist_artwork = artworksfetcher.get_artist_artwork(
            appconfig.Testing.LASTFM_SECRET_API_KEY,
            FetcherArtworkTest.FOLDER,
            'Michael Jackson'
        )
        album_artwork = artworksfetcher.get_album_artwork(
            appconfig.Testing.LASTFM_SECRET_API_KEY,
            FetcherArtworkTest.FOLDER,
            'Thriller',
            'Michael Jackson',
            None
        )

        self.assertTrue(pathlib.Path(artist_artwork).exists())
        self.assertTrue(pathlib.Path(album_artwork).exists())


    def test_lastfm_provider(self):
        provider = artworksfetcher.ProviderLastFM(api_key=appconfig.Testing.LASTFM_SECRET_API_KEY,
                                                  artwork_folder=FetcherArtworkTest.FOLDER)

        artist_artwork = provider.get_artist_artwork('Michael Jackson')
        album_artwork = provider.get_album_artwork('Thriller',
                                                   'Michael Jackson')

        self.assertTrue(pathlib.Path(artist_artwork).exists())
        self.assertTrue(pathlib.Path(album_artwork).exists())

    def test_local_filesystem_provider(self):
        provider = artworksfetcher.ProviderLocalFileSystem()
        album_artwork = provider.get_album_artwork(FetcherArtworkTest.FOLDER)
        self.assertTrue(pathlib.Path(album_artwork).exists())



import shutil
import unittest
import pathlib

from mokaplayer.core.fetchers import tabs as tabsfetcher


class FetcherTabsTest(unittest.TestCase):

    FOLDER = './test/tabs'

    def setUp(self):
        pathlib.Path(FetcherTabsTest.FOLDER).mkdir(exist_ok=True)

    def tearDown(self):
        shutil.rmtree(FetcherTabsTest.FOLDER)

    def test_flow(self):
        tabs = tabsfetcher.search('One', 'Metallica')
        self.assertGreater(len(tabs), 0)
        self.assertIsNotNone(tabs[0])

        guitar_pro = [x for x in tabs if x['type'] == 'guitar pro'].pop()
        ascii_tab = [x for x in tabs if x['type'] == 'tab'].pop()

        ascii_text = tabsfetcher.fetch_ascii_tab(ascii_tab['url'])
        path = tabsfetcher.download_guitar_pro_tab(guitar_pro['url'], FetcherTabsTest.FOLDER)

        self.assertIsNotNone(ascii_text)
        self.assertTrue(pathlib.Path(path).is_file())



import unittest
from collections import deque

from mokaplayer.core.queue import Queue


class QueueTest(unittest.TestCase):
    def setUp(self):
        self.queue = Queue()
        self.queue.append([1, 2, 3, 4, 5, 6, 7, 8, 9])

    def test_peek(self):
        self.assertEqual(self.queue.peek(), 1)

    def test_next(self):
        self.queue.next()
        self.assertEqual(self.queue.peek(), 2)
        self.assertEqual(self.queue._container,
                         deque([2, 3, 4, 5, 6, 7, 8, 9, 1]))

    def test_next_while_empty(self):
        self.queue.clear()
        self.queue.next()
        self.assertEqual(self.queue.peek(), None)

    def test_prev(self):
        self.queue.prev()
        self.assertEqual(self.queue.peek(), 9)
        self.assertEqual(self.queue._container,
                         deque([9, 1, 2, 3, 4, 5, 6, 7, 8]))

    def test_prev_while_empty(self):
        self.queue.clear()
        self.queue.prev()
        self.assertEqual(self.queue.peek(), None)

    def test_seek(self):
        self.queue.seek(6)
        self.assertEqual(self.queue._container,
                         deque([6, 7, 8, 9, 1, 2, 3, 4, 5]))
        self.assertEqual(self.queue.peek(), 6)

        self.queue.seek(86)
        self.assertEqual(self.queue._container, deque(
            [86, 7, 8, 9, 1, 2, 3, 4, 5, 6]))
        self.assertEqual(self.queue.peek(), 86)

    def test_shuffle(self):
        self.queue.shuffle()
        self.assertEqual(self.queue.peek(), 1)
        self.assertNotEqual(self.queue._container,
                            deque([1, 2, 3, 4, 5, 6, 7, 8, 9]))

    def test_shuffle_when_current_not_in_queue(self):
        self.queue.clear()
        self.queue.append([20, 21, 22, 24])
        self.assertEqual(self.queue.peek(), 1)
        self.assertNotIn(1, self.queue._container)

    def test_append(self):
        self.queue.append(77)
        self.queue.append([1, 2, 5, 89, 45, 23])
        self.assertEqual(self.queue._container,
                         deque([3, 4, 6, 7, 8, 9, 77, 1, 2, 5, 89, 45, 23]))
        self.assertEqual(self.queue.peek(), 1)

        self.queue = Queue()
        self.queue.append([5, 6, 7, 8])
        self.assertEqual(self.queue.peek(), 5)

    def test_prepend(self):
        self.queue.prepend(77)
        self.queue.prepend([23, 8, 9, 3])
        self.assertEqual(self.queue._container,
                         deque([1, 23, 8, 9, 3, 77, 2, 4, 5, 6, 7]))
        self.assertEqual(self.queue.peek(), 1)

        self.queue = Queue()
        self.queue.prepend([5, 6, 7, 8])
        self.assertEqual(self.queue.peek(), 5)

    def test_remove(self):
        self.queue.remove(77)
        self.queue.remove([1, 4, 9])
        self.queue.remove(3)
        self.assertEqual(self.queue._container, deque([2, 5, 6, 7, 8]))
        self.assertEqual(self.queue.peek(), 1)

    def test_clear(self):
        self.queue.clear()
        self.assertEqual(len(self.queue), 0)

    def test_len(self):
        self.assertEqual(len(self.queue), 9)

    def test_pop(self):
        self.queue.pop()
        self.assertIsNone(self.queue.peek())
        self.queue.next()
        self.assertIsNotNone(self.queue.peek())

    def test_iter(self):
        for x in self.queue:
            self.assertIsNotNone(x)



import shutil
import time
import uuid
import unittest
import pathlib
from mokaplayer.core.library import Library
from mokaplayer.core.database import Song, Artist, Playlist, Album
from mokaplayer.core.m3u_parser import M3uParser
from mokaplayer.config import appconfig, userconfig


class LibraryTest(unittest.TestCase):

    FOLDER = './test/library/'
    PLAYLIST_FILE = './test/library/playlist.m3u'
    SRC_FILE = './test/data/empty.ogg'
    DST_FILE = [
        './test/library/song1.ogg',
        './test/library/song2.ogg',
        './test/library/song3.ogg',
        './test/library/song4.ogg',
        './test/library/song5.ogg',
        './test/library/song6.ogg',
        './test/library/song7.ogg',
        './test/library/song8.ogg',
    ]

    def setUp(self):
        pathlib.Path(LibraryTest.FOLDER).mkdir(exist_ok=True)

        usrcfg = userconfig.UserConfig()
        usrcfg['library'] = {}
        usrcfg['library']['music_directory'] = self.FOLDER
        usrcfg['library']['playlist_directory'] = self.FOLDER

        self.library = Library(appconfig.Testing, usrcfg)

        (pathlib.Path(self.FOLDER) / 'fake.png').touch()
        (pathlib.Path(self.FOLDER) / 'fakefolder').mkdir()

        # Create 8 songs on 4 different album from 2 different artist
        for index, path in enumerate(LibraryTest.DST_FILE):
            shutil.copy(LibraryTest.SRC_FILE,  path)
            song = Song(Path=path)
            song.read_tags()
            song.Title = 'Title' + str(index)
            song.AlbumArtist = 'Artist' + str(int(index / 4))
            song.Artist = 'Artist' + str(int(index / 4))
            song.Album = 'Album' + str(int(index / 2))
            song.write_tags()

        playlist = M3uParser(LibraryTest.PLAYLIST_FILE)
        for path in LibraryTest.DST_FILE:
            playlist.append(pathlib.Path(path).name)
        playlist.write()


    def tearDown(self):
        shutil.rmtree(LibraryTest.FOLDER)

    def test_musics_folder(self):
        self.assertTrue(self.library.is_musics_folder_valid())
        self.library.musics_folder = 'asdf/Invalid folder/fasdfasdf'
        self.assertEqual(self.library.musics_folder, 'asdf/Invalid folder/fasdfasdf')
        self.assertFalse(self.library.is_musics_folder_valid())
        self.library.musics_folder = ''
        self.assertFalse(self.library.is_musics_folder_valid())
        self.library.musics_folder = self.FOLDER
        self.assertTrue(self.library.is_musics_folder_valid())
        self.assertEqual(self.library.musics_folder, self.FOLDER)

    def test_playlists_folder(self):
        self.assertEqual(self.library.playlists_folder, self.FOLDER)
        self.library.playlists_folder = 'asdf/Invalid folder/fasdfasdf'
        self.assertEqual(self.library.playlists_folder, self.FOLDER)
        self.library.playlists_folder = './test'
        self.assertEqual(self.library.playlists_folder, './test')

    def test_sync(self):
        self.library.sync()

        time.sleep(1)
        self.assertEqual(Song.select().count(), 8)
        self.assertEqual(Artist.select().count(), 2)
        self.assertEqual(Album.select().count(), 4)
        self.assertEqual(Playlist.select().count(), 1)

        self.library.sync()

        self.assertEqual(Song.select().count(), 8)
        self.assertEqual(Artist.select().count(), 2)
        self.assertEqual(Album.select().count(), 4)
        self.assertEqual(Playlist.select().count(), 1)

    def test_sync_with_delete(self):
        self.library.sync()

        for index, path in enumerate(LibraryTest.DST_FILE):
            if index != 0:
                pathlib.Path(path).unlink()

        pathlib.Path(self.PLAYLIST_FILE).unlink()

        self.library.sync()

        self.assertEqual(Song.select().count(), 1)
        self.assertEqual(Album.select().count(), 1)
        self.assertEqual(Artist.select().count(), 1)
        self.assertEqual(Playlist.select().count(), 0)

    def test_sync_with_add(self):
        self.library.sync()

        new_song_path = './test/library/songa.ogg'
        shutil.copy(LibraryTest.SRC_FILE,  new_song_path)

        song = Song(Path=new_song_path)
        song.read_tags()
        song.Title = 'Title!'
        song.Artist = 'Artist!'
        song.Album = 'Album!'
        song.write_tags()

        self.library.sync()

        self.assertEqual(Song.select().count(), 9)
        self.assertEqual(Artist.select().count(), 3)
        self.assertEqual(Album.select().count(), 5)


import unittest
import pathlib
import shutil
import logging
from mokaplayer.core.database import Song


class TagsTest(unittest.TestCase):

    SRC_FILE = './test/data/empty.ogg'
    DST_FILE = './test/song/song.ogg'
    FOLDER = './test/song'

    def setUp(self):
        pathlib.Path(TagsTest.FOLDER).mkdir(exist_ok=True)

    def tearDown(self):
        shutil.rmtree(TagsTest.FOLDER)

    def test_read_invalid_file(self):
        with self.assertLogs(level=logging.ERROR):
            tmp = Song(Path='invalid/99999999/file')
            tmp.read_tags()

    def test_save_invalid_file(self):
        with self.assertLogs(level=logging.ERROR):
            tmp = Song(Path='invalid/99999999/file')
            tmp.write_tags()

    def test_read_tags(self):
        shutil.copy(TagsTest.SRC_FILE,  TagsTest.DST_FILE)
        song = Song(Path=TagsTest.DST_FILE)
        song.read_tags()
        self.assertEqual(song.Title, 'TestTitle')
        self.assertEqual(song.Artist, 'TestArtist')
        self.assertEqual(song.Album, 'TestAlbum')

    def test_write_tags(self):
        shutil.copy(TagsTest.SRC_FILE,  TagsTest.DST_FILE)
        song = Song(Path=TagsTest.DST_FILE)
        song.read_tags()
        song.Title = 'Title 2'
        song.Artist = 'Artist 2'
        song.Album = 'Album 2'
        song.AlbumArtist = 'AlbumArtist 2'
        song.write_tags()
        song.read_tags()
        self.assertEqual(song.Title, 'Title 2')
        self.assertEqual(song.Artist, 'Artist 2')
        self.assertEqual(song.Album, 'Album 2')
        self.assertEqual(song.AlbumArtist, 'AlbumArtist 2')



import unittest
import pathlib
import shutil

from mokaplayer.core.m3u_parser import M3uParser

class M3uPaserTest(unittest.TestCase):
    FOLDER = './test/playlist/'

    def setUp(self):
        pathlib.Path(M3uPaserTest.FOLDER).mkdir(exist_ok=True)
        self.playlist = M3uParser(M3uPaserTest.FOLDER + 'test.m3u')

    def tearDown(self):
        shutil.rmtree(M3uPaserTest.FOLDER)

    def test_append_same_file(self):
        self.playlist.append('song1.mp3')
        self.playlist.append('song1.mp3')
        self.playlist.append('./song1.mp3')
        self.playlist.append('./././song1.mp3')
        self.assertEqual(len(self.playlist), 1)

    def test_readandwrite(self):
        with self.assertLogs():
            self.playlist.read()

        self.assertEqual(len(self.playlist), 0)

        self.playlist.append("song1.mp3")
        self.playlist.append("song2.wav")
        self.playlist.append("song3.ogg")
        self.playlist.append("song4.mp4")

        self.playlist.write()
        self.playlist.read()

        self.assertEqual(len(self.playlist), 4)

        self.playlist.remove("song1.mp3")
        self.assertEqual(len(self.playlist), 3)
        self.playlist.write()
        self.playlist.read()
        self.assertEqual(len(self.playlist), 3)

    def test_nameandlocation(self):
        self.assertFalse(pathlib.Path(self.playlist.location).is_file())

        self.playlist.write()

        old_location = self.playlist.location
        self.assertTrue(pathlib.Path(old_location).is_file())
        self.assertEqual(self.playlist.name, 'test')

        self.playlist.name = 'New Name'

        self.assertEqual(self.playlist.name, 'New Name')
        self.assertIn('New Name.m3u', self.playlist.location)
        self.assertTrue(pathlib.Path(self.playlist.location).is_file())
        self.assertFalse(pathlib.Path(old_location).is_file())



import unittest
import time
import os

from mokaplayer.core.streamer import Streamer

class StreamerTest(unittest.TestCase):

    SILENCE_AUDIO_FILE = os.path.abspath('test/data/silence.mp3')

    def setUp(self):
        self.streamer = Streamer()
        self.streamer.volume = 0

    def test_play_without_stream(self):
        self.streamer.play()
        self.assertEqual(self.streamer.state, self.streamer.State.PLAYING)

    def test_play_with_stream(self):
        self.streamer.play(self.SILENCE_AUDIO_FILE)
        self.assertEqual(self.streamer.state, self.streamer.State.PLAYING)

    def test_stop(self):
        self.streamer.stop()
        self.assertEqual(self.streamer.state, self.streamer.State.STOPED)

    def test_pause(self):
        self.streamer.pause()
        self.assertEqual(self.streamer.state, self.streamer.State.PAUSED)

    def test_resume(self):
        self.streamer.resume()
        self.assertEqual(self.streamer.state, self.streamer.State.PLAYING)

    def test_stream(self):
        self.streamer.stream = self.SILENCE_AUDIO_FILE
        self.assertTrue(self.streamer.stream)

    def test_invalid_stream(self):
        self.streamer.stream = None
        self.assertFalse(self.streamer.stream)

        with self.assertRaises(ValueError):
            self.streamer.stream = 'asdfasdfasdf'

    def test_volume(self):
        self.streamer.volume = -100
        self.assertEqual(self.streamer.volume, 0)
        self.streamer.volume = 100
        self.assertEqual(self.streamer.volume, 1)
        self.streamer.volume = 0.3
        self.assertAlmostEqual(self.streamer.volume, 0.3)

    def test_duration(self):
        self.streamer.play(self.SILENCE_AUDIO_FILE)
        time.sleep(.3)
        self.assertAlmostEqual(self.streamer.duration, 15.264)

    def test_position(self):
        self.streamer.play(self.SILENCE_AUDIO_FILE)
        time.sleep(.3)
        self.assertGreater(self.streamer.position, 0.2)
        self.assertLess(self.streamer.position, 0.4)

    def test_position_jump(self):
        self.streamer.play(self.SILENCE_AUDIO_FILE)
        time.sleep(.3)
        self.streamer.position = 10
        time.sleep(.3)
        self.assertGreater(self.streamer.position, 9.5)
        self.assertLess(self.streamer.position, 10.5)




import unittest
import os
import logging
import pathlib

from mokaplayer.config import appconfig
from mokaplayer.config import userconfig

class UserConfigTest(unittest.TestCase):

    CONFIG_FILE_PATH = appconfig.Testing.USER_CONFIG_FILE

    def setUp(self):
        self.cfg = userconfig.UserConfig(self.CONFIG_FILE_PATH)

    def tearDown(self):
        os.remove(self.CONFIG_FILE_PATH)

    def test_read_invalid_file(self):
        with self.assertLogs(level=logging.CRITICAL):
            tmp = userconfig.UserConfig()
            tmp._path ='invalid/99999999/file'
            tmp.read()

    def test_save_invalid_file(self):
        with self.assertLogs(level=logging.CRITICAL):
            tmp = userconfig.UserConfig()
            tmp._path ='invalid/99999999/file'
            tmp.save()

    def test_get_file(self):
        self.assertIsInstance(self.cfg.get_file(), pathlib.Path)
        self.assertEqual(self.cfg.get_file().resolve(),
                         pathlib.Path(self.CONFIG_FILE_PATH).resolve())

    def test_create(self):
        os.remove(self.CONFIG_FILE_PATH)
        self.cfg.create()
        self.assertTrue(os.path.exists(self.CONFIG_FILE_PATH))

    def test_read(self):
        self.cfg['data'] = 1
        self.cfg.save()
        self.cfg['data'] = 3
        self.cfg.read()
        self.assertEqual(1, self.cfg['data'])

    def test_save(self):
        self.cfg['data'] = 1
        self.assertEqual(1, self.cfg['data'])

        self.cfg.save()
        self.assertEqual(1, self.cfg['data'])

        new_cfg = userconfig.UserConfig(self.CONFIG_FILE_PATH)
        self.assertEqual(1, new_cfg['data'])

    def test_merge(self):
        d1 = {}
        d2 = {'a': {'b': 'c'}, 'd': 1}
        result = {'a': {'b': 'c'}, 'd': 1}

        userconfig.merge(d1, d2)
        self.assertDictEqual(d1, result)

        d1 = {'a': {'b': '2'}, 'y': 3}
        d2 = {'a': {'b': 'c'}, 'd': 1}
        result = {'a': {'b': 'c'}, 'd': 1, 'y': 3}

        userconfig.merge(d1, d2)
        self.assertDictEqual(d1, result)
